#include <stdio.h>
#include <stdlib.h>

#ifndef WIN32
#include <sys/resource.h>
#include <sys/times.h>
#include <unistd.h>
#endif

#include "CSTree.h"
#include "global.h"
#include "context.h"
#include "cat_rtree.h"
#include "disqualify.h"
#include "mdcCompressTree.h"
#include "preferenceTree.h"
#include "sfs.h"


#ifndef WIN32
void calculateExecutionTime(struct rusage *myTime1, struct rusage *myTime2, float *userTime, float *sysTime)
{
	(*userTime) =
		((float) (myTime2->ru_utime.tv_sec  - myTime1->ru_utime.tv_sec)) +
		((float) (myTime2->ru_utime.tv_usec - myTime1->ru_utime.tv_usec)) * 1e-6;
	(*sysTime) =
		((float) (myTime2->ru_stime.tv_sec  - myTime1->ru_stime.tv_sec)) +
		((float) (myTime2->ru_stime.tv_usec - myTime1->ru_stime.tv_usec)) * 1e-6;

}
#endif

// return (in terms of bytes)
int calculateStorage(int noOfNode, int noOfDataPointInS, int noOfDataPointInPTail)
{
	int returnValue;

	returnValue = 20*noOfNode + 4*noOfDataPointInS + 4*noOfDataPointInPTail;

	return returnValue;
}	

int main(int argc, char **args)
{
	config *aConfig;
	char *confFilename = "conf.txt";

	//char *debugFilename = "./output/debug.txt";
	//char *STFilename = "./output/st.txt";
	//char *CSTFilename = "./output/cst.txt";
	//char *contextFilename = "./output/contextResult.txt";
	//char *timeFilename = "./output/time.txt";

	char *debugFilename = "debug.txt";
	char *STFilename = "st.txt";
	char *CSTFilename = "cst.txt";
	char *contextFilename = "contextResult.txt";
	char *timeFilename = "time.txt";


	FILE *debugFilePtr;

	FILE *STFilePtr;
	FILE *CSTFilePtr;
	FILE *contextFilePtr;

	int noOfNumAttr;

	projTableWhole *projTabWhole;

	int dimNo;

	mySet *remainD;
	mySet *globalSkyline;

	myClassSetSetSet *disqualifySet;

	FILE *timeFilePtr;
#ifndef WIN32
	

	float  userTime_findProjected, sysTime_findProjected;
	float  userTime_findGlobal, sysTime_findGlobal;
//	float  userTime_remainD, sysTime_remainD;
//	float  userTime_generateST, sysTime_generateST;
//	float  userTime_compressST, sysTime_compressST;
//	float  userTime_normalQuery, sysTime_normalQuery;
//	float  userTime_partialQuery, sysTime_partialQuery;
//	float  userTime_context, sysTime_context;
//	float  userTime_totalGenerate, sysTime_totalGenerate;

	struct rusage myTime_start;
	struct rusage myTime_findProjected_end;
	struct rusage myTime_findGlobal_end;
//	struct rusage myTime_remainD_end;
//	struct rusage myTime_generateST_end;
//	struct rusage myTime_compressST_end;
//	struct rusage myTime_normalQuery_end;
//	struct rusage myTime_partialQuery_end;
//	struct rusage myTime_context_end;
#endif

	aConfig = config_new();

	debugFilePtr = (FILE *) fopen(debugFilename, "w");
	if (debugFilePtr == NULL)
	{
		printf("File %s cannot be opened!\n", debugFilename);
		exit(0);
	}

	STFilePtr = (FILE *) fopen(STFilename, "w");
	if (STFilePtr == NULL)
	{
		printf("File %s cannot be opened!\n", STFilename);
		exit(0);
	}

	CSTFilePtr = (FILE *) fopen(CSTFilename, "w");
	if (CSTFilePtr == NULL)
	{
		printf("File %s cannot be opened!\n", CSTFilename);
		exit(0);
	}

	contextFilePtr = (FILE *) fopen(contextFilename, "w");
	if (contextFilePtr == NULL)
	{
		printf("File %s cannot be opened!\n", contextFilename);
		exit(0);
	}

	timeFilePtr = (FILE *) fopen(timeFilename, "w");
	if (contextFilePtr == NULL)
	{
		printf("File %s cannot be opened!\n", timeFilename);
		exit(0);
	}

	// read config
	printf("Reading Config. file....\n");
	//confFilename = args[1];
	config_readConfig(confFilename, aConfig);

	// read data
	printf("Reading Data File...\n");
	config_readData(aConfig);

	// read template
	printf("Reading the template ...\n");
	config_readTemplate(aConfig);

#ifndef WIN32
	getrusage(RUSAGE_SELF,&myTime_start);	
#endif	

	// find the projected table
	// assume the last dime is categorical attr.
	printf("Finding the projected table ...\n");
	noOfNumAttr = aConfig->noOfDim - aConfig->noOfCat;
	dimNo = noOfNumAttr;
	projTabWhole = projTable_findProjTable_all(aConfig);
	aConfig->projTabWhole = projTabWhole;

#ifndef WIN32
	getrusage(RUSAGE_SELF,&myTime_findProjected_end);	
#endif	

	if (aConfig->ALG == 5 || aConfig->ALG == 6 || aConfig->ALG == 7)
	{
		printf("Finding disqualifying conditions...\n");
		disqualifySet = global_findQualifyingCondition(aConfig, projTabWhole, noOfNumAttr, dimNo);

		//	disqualify_print(debugFilePtr, disqualifySet);
	}
	else if ((aConfig->ALG == 8) || (aConfig->ALG == 9) || (aConfig->ALG == 10))
	{
		// find global skyline and non-skyline
		printf("Finding the global skyline and non-skylines...\n");
		noOfNumAttr = aConfig->noOfDim - aConfig->noOfCat;
		global_findGlobalSkylineAndNonSkyline(aConfig, projTabWhole, noOfNumAttr, dimNo);

#ifndef WIN32
	//getrusage(RUSAGE_SELF,&myTime_findGlobal_end);	
#endif	

		// find remainD and global skyline
		printf("Finding the remaining D and global skyline....\n");
		remainD = set_new();
		globalSkyline = set_new();

		global_findRemainDAndGlobalSkyline(projTabWhole, remainD, globalSkyline);

		// dummy
		//anRtreeGroup = rtreeGroup_new();
	}
#ifndef WIN32
	getrusage(RUSAGE_SELF,&myTime_findGlobal_end);	
#endif

#ifndef WIN32
//	getrusage(RUSAGE_SELF,&myTime_compressST_end);	
#endif	

	srand(time(NULL));
//		disqualify_print(debugFilePtr, disqualifySet);
	if (aConfig->ALG == 5)
	{
		// MDC Tree
		expandTreeInit(disqualifySet, aConfig, debugFilePtr, timeFilePtr);
	}
	else if (aConfig->ALG == 6)
	{
		// Preference Tree
		preferenceTree_skylineQuery(aConfig, disqualifySet, debugFilePtr, timeFilePtr);
	}
	else if (aConfig->ALG == 7)
	{
		// MDC Table
		disqualifyImplicitPref_main(aConfig, disqualifySet, debugFilePtr, timeFilePtr);
	}
	else if (aConfig->ALG == 8)
	{
		// SFS (adaptive)
		sfs_main(aConfig, remainD, globalSkyline, debugFilePtr, timeFilePtr);
	}
	else if (aConfig->ALG == 9)
	{
		// SFS (based on SKY(R))
		sfsBaseline_main(aConfig, remainD, globalSkyline, debugFilePtr, timeFilePtr);
	}
	else if (aConfig->ALG == 10)
	{
		// SFS (based on D)
		sfsNaive_main(aConfig, remainD, globalSkyline, debugFilePtr, timeFilePtr);
	}

#ifndef WIN32
//	getrusage(RUSAGE_SELF,&myTime_normalQuery_end);	
#endif	

#ifndef WIN32
//	getrusage(RUSAGE_SELF,&myTime_context_end);	
#endif	

#ifndef WIN32

	// findProjected
//	calculateExecutionTime(&myTime_start, &myTime_findProjected_end, &userTime_findProjected, &sysTime_findProjected);
	// Find time needed to build mdc
	//calculateExecutionTime(&myTime_start, &myTime_findGlobal_end, &userTime_findProjected, &sysTime_findProjected);
	calculateExecutionTime(&myTime_start, &myTime_findProjected_end, &userTime_findProjected, &sysTime_findProjected);

	// findGlobal
	calculateExecutionTime(&myTime_findProjected_end, &myTime_findGlobal_end, &userTime_findGlobal, &sysTime_findGlobal);

	// remainD
//	calculateExecutionTime(&myTime_findGlobal_end, &myTime_remainD_end, &userTime_remainD, &sysTime_remainD);

	// generateST
//	calculateExecutionTime(&myTime_remainD_end, &myTime_generateST_end, &userTime_generateST, &sysTime_generateST);

	// compressST
//	calculateExecutionTime(&myTime_generateST_end, &myTime_compressST_end, &userTime_compressST, &sysTime_compressST);

	// normalQuery
//	calculateExecutionTime(&myTime_compressST_end, &myTime_normalQuery_end, &userTime_normalQuery, &sysTime_normalQuery);

	// partialQuery
//	calculateExecutionTime(&myTime_normalQuery_end, &myTime_partialQuery_end, &userTime_partialQuery, &sysTime_partialQuery);

	// context
//	calculateExecutionTime(&myTime_partialQuery_end, &myTime_context_end, &userTime_context, &sysTime_context);

	// totalGenerate
//	calculateExecutionTime(&myTime_start, &myTime_compressST_end, &userTime_totalGenerate, &sysTime_totalGenerate);

	

	// findProjected
//	fprintf(timeFilePtr, "%.5f\t%.5f\n", userTime_findProjected, sysTime_findProjected);
//
//	// findGlobal
//	fprintf(timeFilePtr, "%.5f\t%.5f\n", userTime_findGlobal, sysTime_findGlobal);
//	fprintf(stdout, "finding mdc time: %.5f\t%.5f\n", userTime_findGlobal, sysTime_findGlobal);
//
//	// remainD
//	fprintf(timeFilePtr, "%.5f\t%.5f\n", userTime_remainD, sysTime_remainD);
//
//	// generateST
//	fprintf(timeFilePtr, "%.5f\t%.5f\n", userTime_generateST, sysTime_generateST);
//
//	// compressST
//	fprintf(timeFilePtr, "%.5f\t%.5f\n", userTime_compressST, sysTime_compressST);
//
//	// normalQuery
//	fprintf(timeFilePtr, "%.5f\t%.5f\n", userTime_normalQuery, sysTime_normalQuery);
//	fprintf(stdout, "querying time: %.5f\t%.5f\n", userTime_normalQuery, sysTime_normalQuery);
//
//	// partialQuery
//	fprintf(timeFilePtr, "%.5f\t%.5f\n", userTime_partialQuery, sysTime_partialQuery);
//
//
//	// context
//	fprintf(timeFilePtr, "%.5f\t%.5f\n", userTime_context, sysTime_context);
//
//	// totalGenerate
//	fprintf(timeFilePtr, "%.5f\t%.5f\n", userTime_totalGenerate, sysTime_totalGenerate);
//
	printf("MDC build time: %.10f\t%.10f\n", userTime_findProjected, sysTime_findProjected);
	fprintf(timeFilePtr, "%.10f\t%.10f\n", userTime_findProjected, sysTime_findProjected);

	// findGlobal
	fprintf(timeFilePtr, "%.10f\t%.10f\n", userTime_findGlobal, sysTime_findGlobal);
	fprintf(stdout, "finding mdc time: %.10f\t%.10f\n", userTime_findGlobal, sysTime_findGlobal);
//
//	printf("User time : %.5f seconds\n", userTime_totalGenerate);
//	printf("System time : %.5f seconds\n\n", sysTime_totalGenerate);

	
#endif

	printf("Closing file...\n");
	fclose(timeFilePtr);
	fclose(debugFilePtr);


	printf("Freeing memory...\n");
	config_free(aConfig);
	free(aConfig);

	return 0;
}
