
#include "config.h"
#include "ordering.h"
#include "set.h"

config *config_new()
{
	config *returnValue;

	returnValue = (config *) malloc(sizeof(config));
	memset(returnValue, 0, sizeof(config));
	returnValue->dataFilename = (char *) malloc(sizeof(char)*1024);
	returnValue->tempFilename = (char *) malloc(sizeof(char)*1024);

	returnValue->queryFilename = (char *) malloc(sizeof(char)*1024);
	returnValue->partialFilename = (char *) malloc(sizeof(char)*1024);

	//returnValue->templ = ordering_new();

	return returnValue;
}

void config_free(config *aConfig)
{
	free(aConfig->dataFilename);
	free(aConfig->tempFilename);
	free(aConfig->partialFilename);
	free(aConfig->queryFilename);
	if (aConfig->noOfCat > 0)
	{
		free(aConfig->noOfClass);
	}
	if (aConfig->noOfData > 0)
	{
		free(aConfig->data);
	}
}


void config_readConfig(char *filename, config *aConfig)
{
	FILE *filePtr;

	int i;

	char *tempBuffer;

	filePtr = (FILE *) fopen(filename, "r");
	if (filePtr == NULL)
	{
		printf("File %s does not exist!\n", filename);
		exit(0);
	}

	tempBuffer = (char *) malloc(sizeof(char)*1024);

	fscanf(filePtr, "%d\n", &(aConfig->noOfData));
	fscanf(filePtr, "%d\n", &(aConfig->noOfDim));
	fscanf(filePtr, "%d\n", &(aConfig->noOfCat));
	fscanf(filePtr, "%d\n", &(aConfig->nl));
	fscanf(filePtr, "%d\n", &(aConfig->ALG));
	fscanf(filePtr, "\n");
	fscanf(filePtr, "%d\n", &(aConfig->m));
	fscanf(filePtr, "%d\n", &(aConfig->M));
	fscanf(filePtr, "%d\n", &(aConfig->reinsert_p));
	fscanf(filePtr, "\n");
	fscanf(filePtr, "%s\n", tempBuffer);
	strcpy(aConfig->dataFilename, tempBuffer);
	fscanf(filePtr, "%s\n", tempBuffer);
	strcpy(aConfig->tempFilename, tempBuffer);
	fscanf(filePtr, "%s\n", tempBuffer);
	strcpy(aConfig->queryFilename, tempBuffer);
	fscanf(filePtr, "%s\n", tempBuffer);
	strcpy(aConfig->partialFilename, tempBuffer);

	//new added
	//for multi-query
	fscanf(filePtr, "%d\n", &(aConfig->noOfFullOrderQuery));
	fscanf(filePtr, "%d\n", &(aConfig->noOfPartialOrderQuery));
	fscanf(filePtr, "%d\n", &(aConfig->isCompress));
	fscanf(filePtr, "%d\n", &(aConfig->compress_noOfClass));
	fscanf(filePtr, "\n");

	free(tempBuffer);

	aConfig->noOfClass = (int *) malloc(sizeof(int)*aConfig->noOfCat);
	memset(aConfig->noOfClass, 0, sizeof(int)*aConfig->noOfCat);

	for (i = 0; i < aConfig->noOfCat; i++)
	{
		fscanf(filePtr, "%d\n", &(aConfig->noOfClass[i]));
	}

	fclose(filePtr);
}

void config_readData(config *aConfig)
{
	FILE *filePtr;

	int noOfData, noOfDim;


	int i, j;

	record *tempRecord;

	filePtr = (FILE *) fopen(aConfig->dataFilename, "r");
	if (filePtr == NULL)
	{
		printf("File %s does not exist!\n", aConfig->dataFilename);
		exit(0);
	}

	noOfData = aConfig->noOfData;
	noOfDim = aConfig->noOfDim;

	tempRecord = (record *) malloc(sizeof(record)*noOfData);

	for (i = 0; i < noOfData; i++)
	{
		tempRecord[i].content = (int *) malloc(sizeof(int)*noOfDim);
		for (j = 0; j < noOfDim; j++)
		{
			fscanf(filePtr, "%d ", &(tempRecord[i].content[j]));
		}
	}

	aConfig->data = tempRecord;

	fclose(filePtr);
}

void config_readTemplate(config *aConfig)
{
	FILE *filePtr;

	int noOfBinaryOrder;

	int i, j;

	ordering *templ;
	ordering **templList;

	int classBefore, classAfter;

	binaryOrder *orderList;

	int noOfNumAttr;

	//char *tempFilename;
	int anIndex;

	noOfNumAttr = aConfig->noOfDim - aConfig->noOfCat;

	//tempFilename = (char *) malloc(sizeof(char)*1024);

	filePtr = (FILE *) fopen(aConfig->tempFilename, "r");

	if (filePtr == NULL)
	{
		printf("File %s does not exist!\n", aConfig->tempFilename);
		exit(0);
	}


	templList = (ordering **) malloc(sizeof(ordering *)*aConfig->noOfCat);

	aConfig->templ = templList;

	for (j = 0; j < aConfig->noOfCat; j++)
	{
		if (j != 0)
		{
			fscanf(filePtr, "\n");
		}
		templList[j] = ordering_new();

		templ = aConfig->templ[j];

		anIndex = noOfNumAttr+j;
		//sprintf(tempFilename, "%s.%d", aConfig->tempFilename, anIndex);

		fscanf(filePtr, "%d\n", &noOfBinaryOrder);

		templ->noOfBinaryOrder = noOfBinaryOrder;
		orderList = (binaryOrder *) malloc(sizeof(binaryOrder)*noOfBinaryOrder);
		memset(orderList, 0, sizeof(binaryOrder)*noOfBinaryOrder);
		templ->orderList = orderList;

		for (i = 0; i < noOfBinaryOrder; i++)
		{
			fscanf(filePtr, "%d %d\n", &classBefore, &classAfter);

			orderList[i].classBefore = classBefore;
			orderList[i].classAfter = classAfter;
		}

		
	}

	fclose(filePtr);

	//free(tempFilename);

}

/**
 * @author Terry & Sing
 * @since 21 Nov 2006
 */
myOrderSet *config_readOneQuery(FILE *filePtr, int noOfOrdering,
int noOfNominalAttribute, int noOfDim)
{
	myOrderSet *returnValue;

	int i, j;

	mySet *anOrdering;

	int dimNo;
	int noOfNumAttr;

	int aNo;

	returnValue = orderSet_new();

	noOfNumAttr = noOfDim - noOfNominalAttribute;

	if (filePtr == NULL)
	{
		puts("Long query file does not exist!\n");
		exit(0);
	}

	for (i = 0; i < noOfNominalAttribute; i++)
	{
//		fscanf(filePtr, "%d\n", &noOfOrdering);

		anOrdering = set_new();
		dimNo = noOfNumAttr+i;
		orderSet_insert(returnValue, dimNo, &(anOrdering));

		for (j = 0; j < noOfOrdering; j++)
		{
			fscanf(filePtr, "%d\n", &aNo);

			set_insert(anOrdering, aNo);
		}
	}
//	fclose(filePtr);
	return returnValue;
}

myOrderSet *config_readAnQuery(config *aConfig, int isRand)
{
	myOrderSet *returnValue;

	FILE *filePtr;

	int i, j;
	int noOfQuery;
	int queryNo;

	int noOfOrdering;
	int noOfNumber;

	mySet *anOrdering;

	int dimNo;
	int noOfNumAttr;

	int aNo;

	returnValue = orderSet_new();

	noOfNumAttr = aConfig->noOfDim - aConfig->noOfCat;

	filePtr = (FILE *) fopen(aConfig->queryFilename, "r");
	if (filePtr == NULL)
	{
		printf("File %s does not exist!\n", aConfig->queryFilename);
		exit(0);
	}

	fscanf(filePtr, "%d\n", &noOfQuery);
	fscanf(filePtr, "%d\n", &noOfNumber);
	if (isRand == 1)
	{
		queryNo = (int)(rand()/(RAND_MAX+1.0)*noOfQuery);
	}
	else
	{	
		queryNo = 0;
	}
	//printf("\nquery no: %d\n", queryNo);

	//go to the queryNo line
	for (i=0; i<queryNo; i++)
	{
		for (j=0; j<noOfNumber; j++)
		{
			fscanf(filePtr, "%*s"); 
		}
	}

	for (i = 0; i < aConfig->noOfCat; i++)
	{
		fscanf(filePtr, "%d", &noOfOrdering);
		//printf("no of ordering: %d\n", noOfOrdering);

		anOrdering = set_new();
		dimNo = noOfNumAttr+i;
		orderSet_insert(returnValue, dimNo, &(anOrdering));

		//printf("order: ");
		for (j = 0; j < noOfOrdering; j++)
		{
			fscanf(filePtr, "%d", &aNo);
			//printf("%d, ", aNo);

			set_insert(anOrdering, aNo);
		}
		//printf("\n");
	}

	fclose(filePtr);

	return returnValue;
}

myOrderSet *config_readQuery(config *aConfig)
{
	myOrderSet *returnValue;

	FILE *filePtr;

	int i, j;

	int noOfOrdering;

	mySet *anOrdering;

	int dimNo;
	int noOfNumAttr;

	int aNo;

	returnValue = orderSet_new();

	noOfNumAttr = aConfig->noOfDim - aConfig->noOfCat;

	filePtr = (FILE *) fopen(aConfig->queryFilename, "r");
	if (filePtr == NULL)
	{
		printf("File %s does not exist!\n", aConfig->queryFilename);
		exit(0);
	}

	for (i = 0; i < aConfig->noOfCat; i++)
	{
		fscanf(filePtr, "%d\n", &noOfOrdering);

		anOrdering = set_new();
		dimNo = noOfNumAttr+i;
		orderSet_insert(returnValue, dimNo, &(anOrdering));

		for (j = 0; j < noOfOrdering; j++)
		{
			fscanf(filePtr, "%d\n", &aNo);

			set_insert(anOrdering, aNo);
		}
	}

	fclose(filePtr);

	return returnValue;
}

/**
 */
mySetSet *config_readPreference(config *aConfig)
{
	mySetSet *returnValue;

	FILE *filePtr;

	int i, j;

	int noOfOrdering;

	mySet *anOrdering;

	int dimNo;
	int noOfNumAttr;

	int aNo;

	returnValue = setset_new();

	noOfNumAttr = aConfig->noOfDim - aConfig->noOfCat;

	filePtr = (FILE *) fopen(aConfig->queryFilename, "r");
	if (filePtr == NULL)
	{
		printf("File %s does not exist!\n", aConfig->queryFilename);
		exit(0);
	}

	for (i = 0; i < aConfig->noOfCat; i++)
	{
		fscanf(filePtr, "%d\n", &noOfOrdering);

		anOrdering = set_new();
		dimNo = noOfNumAttr+i;
		setset_insert(returnValue, &(anOrdering));

		for (j = 0; j < noOfOrdering; j++)
		{
			fscanf(filePtr, "%d\n", &aNo);

			set_insert(anOrdering, aNo);
		}
	}

	fclose(filePtr);

	return returnValue;
}

mainOrdering *config_readPartialOrder(config *aConfig)
{
	mainOrdering *returnValue;

	FILE *filePtr;
	int i, j;

	int noOfBinaryOrder;

	int classBefore, classAfter;

	ordering *anOrdering;

	returnValue = mainOrdering_new();

	filePtr = (FILE *) fopen(aConfig->partialFilename, "r");
	if (filePtr == NULL)
	{
		printf("File %s does not exist!\n", aConfig->partialFilename);
		exit(0);
	}

	for (i = 0; i < aConfig->noOfCat; i++)
	{
		if (i != 0)
		{
			fscanf(filePtr, "\n");
		}

		anOrdering = ordering_new();

		fscanf(filePtr, "%d", &noOfBinaryOrder);
		for (j = 0; j < noOfBinaryOrder; j++)
		{
			fscanf(filePtr, "%d %d\n", &(classBefore), &(classAfter));
			ordering_insertBinaryOrder2Ordering(anOrdering, classBefore, classAfter);
		}

		mainOrdering_insert(returnValue, &anOrdering);
	}

	fclose(filePtr);


	return returnValue;
	
}

projTable *projTable_new()
{
	projTable *returnValue;

	returnValue = (projTable *) malloc(sizeof(projTable));
	memset(returnValue, 0, sizeof(projTable));

	return returnValue;
}

void projTable_free(projTable *projTab)
{
	int i;

	record *data;
	if (projTab->noOfData > 0)
	{
		free(projTab->isMark);
		free(projTab->pointID);

		data = projTab->data;

		for (i = 0; i < projTab->noOfData; i++)
		{
			free(data[i].content);
		}
		free(projTab->data);
	}
}

void projTableWhole_free(projTableWhole *projTabWhole)
{
	int i;
	
	for (i = 0; i < projTabWhole->noOfProjTable; i++)
	{
		projTable_free(&(projTabWhole->projTab[i]));
	}
	free(projTabWhole->projTab);
}


myClassSetSet *projTable_findAllPossClass(config *aConfig)
{
	myClassSetSet *returnValue;

	int i, j;
	record *data;
	int noOfData;

	int noOfNumAttr;
	myClassSet *aClassSet;

	int index;
	int dimNo, classID;

	int isExist;

	returnValue = classSetSet_new();

	data = aConfig->data;
	noOfData = aConfig->noOfData;

	noOfNumAttr = aConfig->noOfDim - aConfig->noOfCat;

	
	for (i = 0; i < noOfData; i++)
	{
		aClassSet = classSet_new();

		for (j = 0; j < aConfig->noOfCat; j++)
		{
			index = noOfNumAttr + j;
			dimNo = index; //data[i].content[index];
			classID = data[i].content[index];
			classSet_insert(aClassSet, dimNo, classID);
		}

//if (i == 3)
//{
//	printf("");
//}
		isExist = classSetSet_insertNonRedundant(returnValue, &aClassSet);


		if (isExist == 1)
		{
			classSet_free(aClassSet);
			free(aClassSet);
		}
	}
	

	return returnValue;
}

int *projTable_findUniqueIDIndex(config *aConfig, myClassSetSet *allPossClass)
{
	int *returnValue;

	int noOfData;
	record *data;

	int i, j;

	int noOfClass;

	int dimNo;

	int noOfNumAttr;

	myClassSet **classList;

	myClassSet *tempClassSet;

	int isEqual;
	int index;

	int classID;

	int isFound;

	noOfData = aConfig->noOfData;
	data = aConfig->data;

	returnValue = (int *) malloc(sizeof(int)*noOfData);
	memset(returnValue, 0, sizeof(int)*noOfData);

	noOfClass = allPossClass->noOfElt;
	classList = allPossClass->elt;

	noOfNumAttr = aConfig->noOfDim-aConfig->noOfCat;


	for (i = 0; i < noOfData; i++)
	{
		
		tempClassSet = classSet_new();

		for (j = 0; j < aConfig->noOfCat; j++)
		{
			index = noOfNumAttr + j;
			//dimNo = data[i].content[index];
			dimNo = index;
			classID = data[i].content[index];
			classSet_insert(tempClassSet, dimNo, classID);
		}

		isFound = 0;
		for (j = 0; j < noOfClass; j++)
		{
			isEqual = classSet_isContainAll(classList[j], tempClassSet);

			if (isEqual == 1)
			{
				returnValue[i] = j;
				isFound = 1;
				break;
			}
		}

		if (isFound == 0)
		{
			printf("Has problem!\n");
			returnValue[i] = -1;
		}

		classSet_free(tempClassSet);
		free(tempClassSet);
	}

	return returnValue;
}

projTableWhole *projTable_findProjTable(config *aConfig, int dimNo)
{
	projTableWhole *returnValue;
	projTable *projTab;
	int i;

	int  noOfData;
	int *noOfTuple;
	int *tempNoOfTuple;

	int noOfClass;
	int classId;

	int temp;

	record *data;

	int noOfNumAttr;

	int classID;

	myClassSet *projDimClass;

	noOfData = aConfig->noOfData;

	noOfNumAttr = aConfig->noOfDim-aConfig->noOfCat;
	noOfClass = aConfig->noOfClass[dimNo-noOfNumAttr];

	data = aConfig->data;

	noOfTuple = (int *) malloc(sizeof(int)*noOfClass);
	memset(noOfTuple, 0, sizeof(int)*noOfClass);

	tempNoOfTuple = (int *) malloc(sizeof(int)*noOfClass);
	memset(tempNoOfTuple, 0, sizeof(int)*noOfClass);

	projTab = (projTable *) malloc(sizeof(projTable)*noOfClass);



	returnValue = (projTableWhole *) malloc(sizeof(projTableWhole));
	returnValue->noOfProjTable = noOfClass;
	returnValue->projTab = projTab;
	
	// count no. of tuples
	for (i = 0; i < noOfData; i++)
	{
		classId = data[i].content[dimNo];
		
		noOfTuple[classId]++;
	}

	// malloc
	for (i = 0; i < noOfClass; i++)
	{
		projTab[i].noOfData = noOfTuple[i];
		projTab[i].classNo = i;
		projTab[i].dimNo = dimNo;
		projTab[i].data = (record *) malloc(sizeof(record)*noOfTuple[i]);
		memset(projTab[i].data, 0, sizeof(record)*noOfTuple[i]);

		projTab[i].isMark = (int *) malloc(sizeof(int)*noOfTuple[i]);
		memset(projTab[i].isMark, 0, sizeof(record)*noOfTuple[i]);

		projTab[i].pointID = (int *) malloc(sizeof(int)*noOfTuple[i]);
		memset(projTab[i].pointID, 0, sizeof(record)*noOfTuple[i]);

		classID = i;
		projDimClass = (myClassSet *) classSet_new();
		classSet_insert(projDimClass, dimNo, classID);
		projTab[i].projDimClass = projDimClass;
	}

	// insert to projected
	for (i = 0; i < noOfData; i++)
	{
		classId = data[i].content[dimNo];

		// insert
		temp = tempNoOfTuple[classId];
		projTab[classId].data[temp].content = (int *) malloc(sizeof(int)*aConfig->noOfDim);
		memcpy(projTab[classId].data[temp].content, data[i].content, sizeof(int)*aConfig->noOfDim);

		projTab[classId].pointID[temp] = i;
		
		tempNoOfTuple[classId]++;
	}

	free(noOfTuple);
	free(tempNoOfTuple);

	return returnValue;
}



projTableWhole *projTable_findProjTable_all(config *aConfig)
{
	projTableWhole *returnValue;
	projTable *projTab;
	int i;

	int  noOfData;
	int *noOfTuple;
	int *tempNoOfTuple;

	int noOfClass;
	//int classId;

	int temp;

	record *data;

	int noOfNumAttr;

	int classID;

	myClassSet *projDimClass;

	myClassSetSet *allPossClass;
	int *uniqueClassID;

	noOfData = aConfig->noOfData;

	noOfNumAttr = aConfig->noOfDim-aConfig->noOfCat;
	//noOfClass = aConfig->noOfClass[dimNo-noOfNumAttr];
	allPossClass = projTable_findAllPossClass(aConfig);
	noOfClass = allPossClass->noOfElt;

	data = aConfig->data;

	noOfTuple = (int *) malloc(sizeof(int)*noOfClass);
	memset(noOfTuple, 0, sizeof(int)*noOfClass);

	tempNoOfTuple = (int *) malloc(sizeof(int)*noOfClass);
	memset(tempNoOfTuple, 0, sizeof(int)*noOfClass);

	projTab = (projTable *) malloc(sizeof(projTable)*noOfClass);



	returnValue = (projTableWhole *) malloc(sizeof(projTableWhole));
	returnValue->noOfProjTable = noOfClass;
	returnValue->projTab = projTab;

	// find the uniqueClassID
	uniqueClassID = projTable_findUniqueIDIndex(aConfig, allPossClass);
	
	// count no. of tuples
	for (i = 0; i < noOfData; i++)
	{
		//classId = data[i].content[dimNo];
		classID = uniqueClassID[i];
		
		noOfTuple[classID]++;
	}

	// malloc
	for (i = 0; i < noOfClass; i++)
	{
		projTab[i].noOfData = noOfTuple[i];
		projTab[i].classNo = i;
		//projTab[i].dimNo = dimNo;
		projTab[i].data = (record *) malloc(sizeof(record)*noOfTuple[i]);
		memset(projTab[i].data, 0, sizeof(record)*noOfTuple[i]);

		projTab[i].isMark = (int *) malloc(sizeof(int)*noOfTuple[i]);
		memset(projTab[i].isMark, 0, sizeof(int)*noOfTuple[i]);

		projTab[i].pointID = (int *) malloc(sizeof(int)*noOfTuple[i]);
		memset(projTab[i].pointID, 0, sizeof(int)*noOfTuple[i]);

		//classID = i;
		classID = uniqueClassID[i];
		projDimClass = (myClassSet *) classSet_new();
		//classSet_insert(projDimClass, dimNo, classID);
		classSet_copy(projDimClass, allPossClass->elt[i]);
		projTab[i].projDimClass = projDimClass;
	}

	// insert to projected
	for (i = 0; i < noOfData; i++)
	{
		//classId = data[i].content[dimNo];
		classID = uniqueClassID[i];

		// insert
		temp = tempNoOfTuple[classID];
		projTab[classID].data[temp].content = (int *) malloc(sizeof(int)*aConfig->noOfDim);
		memcpy(projTab[classID].data[temp].content, data[i].content, sizeof(int)*aConfig->noOfDim);

		projTab[classID].pointID[temp] = i;
		
		tempNoOfTuple[classID]++;
	}

	free(noOfTuple);
	free(tempNoOfTuple);

	free(uniqueClassID);

	classSetSet_free(allPossClass);
	free(allPossClass);

	return returnValue;
}

