
#ifndef CONF_H
#define CONF_H

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "set.h"
#include "ordering.h"

struct record_s
{
	int *content;
};

typedef struct record_s record;

typedef struct projTableWhole_s projTableWhole;

struct config_s
{
	int noOfData;
	int noOfDim;
	int noOfCat;
	int noOfFullOrderQuery;
	int noOfPartialOrderQuery;

	int *noOfClass;
	record *data;

	char *dataFilename;
	char *tempFilename;

	char *queryFilename;
	char *partialFilename;

	int nl;

	ordering **templ;

	int ALG; 
	// 1 : brute force
	// 2 : single R-tree
	// 3 : multiple R-tree

	projTableWhole *projTabWhole;

	int m;
	int M;
	int reinsert_p;

	
	// for compression
	int isCompress;
	int compress_noOfClass;
};

typedef struct config_s config;

struct projTable_s
{
	int noOfData;
	int classNo;
	int dimNo;

	record *data;
	int *isMark;	// 1: global skyline, 2: remain skyline, 3: promising non-skyline

	int *pointID;

	myClassSet *projDimClass;
};

typedef struct projTable_s projTable;

struct projTableWhole_s
{
	int noOfProjTable;
	projTable *projTab;
};



config *config_new();
void config_free(config *aConfig);
void config_readConfig(char *filename, config *aConfig);
void config_readData(config *aConfig);
void config_readTemplate(config *aConfig);
projTableWhole *projTable_findProjTable(config *aConfig, int dimNo);

projTableWhole *projTable_findProjTable_all(config *aConfig);


myClassSetSet *projTable_findAllPossClass(config *aConfig);
int *projTable_findUniqueIDIndex(config *aConfig, myClassSetSet *allPossClass);

myOrderSet *config_readQuery(config *aConfig);
mainOrdering *config_readPartialOrder(config *aConfig);
myOrderSet *config_readOneQuery(FILE *filePtr, int noOfOrdering,
		int noOfNominalAttribute, int noOfDim);
myOrderSet *config_readAnQuery(config *aConfig, int isRand);
mySetSet *config_readPreference(config *aConfig);
#endif
