#include "mdcCompressTree.h"

int noOfNode;
int noOfVisitedNode;
/**
 * It pint recurisvely the MDC compression tree
 * @param filePtr
 * @param node The root node
 * @author Terry & Sing
 */
void printMdcCompressTree(FILE *filePtr, mdcTreeNode *node, int *id)
{
	int i=0;
	int selfId;

	(*id)++;
	selfId = *id;
	fprintf(filePtr, "%d [shape=box,label=\"", *id);
	printNodeInfomation(filePtr, node);
	fputs("\"];\n", filePtr);
	for (i=0; i< node->noOfChild; i++)
	{
		fprintf(filePtr, "%d -> %d;\n", selfId, *id+1);
		printMdcCompressTree(filePtr, node->child[i], id);
	}
}

/**
 * It print the refinement and disqualified point of a node
 * @param filePtr file to print to 
 * @param node The node you want to print
 * @author Terry & Sing
 */
void printNodeInfomation(FILE *filePtr, mdcTreeNode *node)
{
	int i=0;
	if (node->parent==NULL)
	{
		fprintf(filePtr, "root:");
	}
	else
	{
		fprintf(filePtr, "(%d, %d, %d):\\r", node->dimNo,
				node->classBefore, node->classAfter);
	}
	fputs("{", filePtr);
	for (i=0; i< node->disqualifiedPoint->noOfElt; i++)
	{
		if (i==node->disqualifiedPoint->noOfElt-1)
		{
			fprintf(filePtr, "%d", node->disqualifiedPoint->elt[i]);
		}
		else 
		{
			fprintf(filePtr, "%d, ", node->disqualifiedPoint->elt[i]);
		}
	}
	fputs("}", filePtr);
}

/**
 * It put all disqualifying condition into a set.
 * @param disqualifySet Table of MDC of all point
 * @return Set of all refinement in MDC
 * @author Terry & Sing
 */
myClassSet* getY(myClassSetSetSet* disqualifySet)
{
	int i=0;
	int j=0;
	myClassSet* y = classSet_new(); 
	for (i = 0; i < disqualifySet->noOfElt; i++)
	{
		for (j = 0; j < disqualifySet->elt[i]->noOfElt; j++)
		{
			classSet_unionNonRedundant2(y,
					disqualifySet->elt[i]->elt[j]);
		}
	}
	return y;
}

/**
 * It attach child as one of child node of parent.
 * @param parent The parent node
 * @param child The child node
 * @author Terry & Sing
 */
void mdcTreeNode_appendChild(mdcTreeNode *parent, mdcTreeNode *child)
{
	if (parent->noOfChild==0)
	{
		parent->child=(mdcTreeNode**)malloc(sizeof(mdcTreeNode*));
	}
	else
	{
		parent->child=(mdcTreeNode**)realloc(parent->child,
				sizeof(mdcTreeNode*)*(parent->noOfChild+1));
	}
	parent->child[parent->noOfChild] = child;
	parent->noOfChild++;
}

/**
 * Allocate and set zero the memory of a new mdcTreeNode.
 * It will also assign the paren node and the refinement
 * correspond to this node. It will also create new mySet
 * to store the disqualified pointID
 * @param parent
 * @param dimNo
 * @param classBefore
 * @param classAfter
 * @return Pointer to the new mdcTreeNode struct
 * @author Terry & Sing
 */
mdcTreeNode* mdcTreeNode_new(mdcTreeNode *parent, int dimNo,
		int classBefore, int classAfter)
{
	mdcTreeNode* node;
	node = (mdcTreeNode*) malloc(sizeof(mdcTreeNode));
	node->parent = parent;
	node->dimNo = dimNo;
	node->classBefore = classBefore;
	node->classAfter = classAfter;
	node->noOfChild = 0;
	node->disqualifiedPoint = set_new();
	node->child = NULL;
	return node;
}

/**
 * Alogrithm for expansion of node
 * @param node
 * @param mdc
 * @param y <b>Assumption:</b>Each mySet member holds
 * only 2 integers
 * @param prefix <b>Assumption:</b>Each mySet member holds
 * only 2 integers
 * @author Terry & Sing
 */

void expandTree(mdcTreeNode *node, myClassSetSetSet *mdc,
		myClassSet *y, myClassSet *prefix)
{
	int i, j, k, l;
	int* isRemoveMdc;
	myClassSet* newPrefix;
	int noOfRemoveMdc;
	mySet* disqualifiedPoint;
	myClassSetSetSet* newMdc;
	myClassSet *newY;

	noOfNode++;

	i=0;
	j=0;
	k=0;
	l=0;
	for (i=0; i< y->noOfElt; i++)
	{

		//Create new node
		mdcTreeNode* newNode = mdcTreeNode_new(node, y->dimNo[i],
				y->list[i]->elt[0], y->list[i]->elt[1]);
		mdcTreeNode_appendChild(node, newNode);

		//Create a new prefix'
		newPrefix = mdcTree_updatePrefix(prefix, y, i);

		//Find set of disqualified point
		isRemoveMdc = (int*)malloc(sizeof(int)*mdc->noOfElt);
		memset(isRemoveMdc, 0, sizeof(int)*mdc->noOfElt);
		noOfRemoveMdc=0;
		disqualifiedPoint = newNode->disqualifiedPoint;
		for (j=0; j< mdc->noOfElt; j++)
		{
			for (k=0; k< mdc->elt[j]->noOfElt; k++)
			{
				if (classSet_isEqualOrIsWeaker(mdc->elt[j]->elt[k], newPrefix))
				{
					set_insert(disqualifiedPoint, mdc->pointID[j]);
					isRemoveMdc[j] = 1;
					noOfRemoveMdc++;
				}
			}
		}

		//Create a new mdc'
		// can combine to above for loop j
		newMdc = classSetSetSet_new();
		for (j=0; j< mdc->noOfElt; j++)
		{
			if (isRemoveMdc[j]==0)
			{
				classSetSetSet_insert(newMdc, mdc->pointID[j], &(mdc->elt[j]));
			}
		}

		//Create a new Y'
		newY = classSet_new();
		for (j=0; j< y->noOfElt; j++)
		{
			int isRemove = 0;
			if (isInconsist(y, j, newPrefix))
			{
				isRemove = 1;
			}
			else if (classSet_isLessThanOrEqual(y, j, y, i))
			{
				isRemove = 1;
			}
			else
			{
				myClassSet *tempPrefix = classSet_new();
				classSet_insertSet(tempPrefix, y->dimNo[j], y->list[j]);
				classSet_unionNonRedundant2(tempPrefix, newPrefix);

				isRemove = 1;
				for (k=0; k< newMdc->noOfElt && isRemove==1; k++)
				{
					for (l=0; l< newMdc->elt[k]->noOfElt && isRemove==1; l++)
					{
						if (classSet_isEqualOrIsWeaker(newMdc->elt[k]->elt[l],
									tempPrefix))
						{
							isRemove = 0;
						}
					}
				}
				free(tempPrefix->dimNo);
				free(tempPrefix->list);
				free(tempPrefix);
			}
			if (isRemove==0)
			{
				classSet_insertSet(newY, y->dimNo[j], y->list[j]);
			}

		}
		//Call expandTree()
		expandTree(newNode, newMdc, newY, newPrefix);

		//free mdc' & y' & prefix'
		//No need to free all members of newMdc because 
		//the members are inserted by reference
		free(newMdc->pointID);
		free(newMdc->elt);
		free(newMdc);
		free(newY->dimNo);
		free(newY->list);
		free(newY);
		free(newPrefix->dimNo);
		free(newPrefix->list);
		free(newPrefix);
	}
}

/**
 * Algorithm for generating compressed tree
 * of all points
 * @param mdc MDC table
 * @param aConfig Config
 * @author Terry & Sing
 */
void expandTreeInit(myClassSetSetSet *mdc, config *aConfig, FILE *debugFilePtr, FILE *timeFilePtr)
{
	int i=0;
	//int id=0;
	//int noOfQuery = 0;
//	int noOfCat = 0;
//	char *longQueryFile = "./experiment/query.long.500k.1d.txt";
#ifndef WIN32
	struct rusage tree_build_start;
	struct rusage tree_build_end;
	struct rusage tree_search_end;
	float qc_tree_build_time_u;
	float qc_tree_build_time_s;
	float qc_tree_search_time_u;
	float qc_tree_search_time_s;
	//FILE *filePtr;
#endif

	mainOrdering *R;
	//myOrderSet * anOrderSet;
	mySetSet *RR;

	mySet *skylineSet;

	mdcTreeNode *root;
	root = mdcTreeNode_new(NULL, -1, -1, -1);
	puts("Building MDC compress tree");
#ifndef WIN32
	getrusage(RUSAGE_SELF,&tree_build_start);
#endif
	buildMdcTree(root, mdc);
#ifndef WIN32
	getrusage(RUSAGE_SELF,&tree_build_end);
#endif
/*
	mySet *allMdcPoint;
	myClassSet *y;
	myClassSet *prefix;
	//No parent for root
	y = getY(mdc);
//	puts("Y:");
//	classSet_print(stdout, y);
	prefix = classSet_new();
	//Assign Sd of root with data points disqualified by any refinement
	//A point is disqualified by any refinement if it cannot be found 
	//in MDC
	allMdcPoint = getAllPointInMdc(mdc);
	for (i=0; i< aConfig->noOfData; i++)
	{
		if (!set_isContain(allMdcPoint, i))
		{
			set_insert(root->disqualifiedPoint, i);
		}
	}
	set_free(allMdcPoint);
	puts("expanding the tree");
	expandTree(root, mdc, y, prefix);
	printf("No of node: %d\n", noOfNode);
	*/


	//printMdcCompressTree(stdout, root, &id);
	puts("Finding skyline...");
	
//	for (i=0; i<aConfig->noOfFullOrderQuery; i++)
//	{
//		anOrderSet = config_readAnQuery(aConfig, 
//			(aConfig->noOfFullOrderQuery==1)?0:1);
//		R = mainOrdering_transformOrderSet2MainOrdering(anOrderSet);
//		skylineSet = mdcTree_findSkyline(aConfig, root, R);
//		//printf("skyline: ");
//		//set_print(stdout, skylineSet);
//		//printf("\n");
//
//		set_free(skylineSet);
//		free(skylineSet);
//		orderSet_free(anOrderSet);
//		free(anOrderSet);
//
//		//free(R);
//		
//		//putchar('a');
//		//fflush(stdout);
//	}
	
	RR = config_readPreference(aConfig);
	R = mainOrdering_transformPartialOrder2MainOrdering(aConfig->noOfClass, RR);
	for (i=0; i<aConfig->noOfFullOrderQuery; i++)
	{
		skylineSet = mdcTree_findSkyline(aConfig, root, R);
		//printf("(%d) ", skylineSet->noOfElt);
//		set_print(stdout, skylineSet);


		//printf("Skyline: ");
		//set_print(stdout, skylineSet);
		//printf("\n");

		set_free(skylineSet);
		free(skylineSet);
	}
#ifndef WIN32
	getrusage(RUSAGE_SELF,&tree_search_end);
#endif

	//print out the informations
	printf("----------");
	printf("no of Node: %d\n", noOfNode);
	printf("size of MDC compressed Tree: %d\n",
			calculateSizeOfMdcCompressTree(root));

	fprintf(debugFilePtr, "%d\n", noOfNode);
	fprintf(debugFilePtr, "%d\n",
			calculateSizeOfMdcCompressTree(root));

//	printf("skyline: ");
//	set_print(stdout, skylineSet);
	printf("\n");
#ifndef WIN32
	calculateExecutionTime(&tree_build_start, &tree_build_end,
			&qc_tree_build_time_u, &qc_tree_build_time_s);
	calculateExecutionTime(&tree_build_end, &tree_search_end,
			&qc_tree_search_time_u, &qc_tree_search_time_s);

	qc_tree_search_time_u /= aConfig->noOfFullOrderQuery;
	qc_tree_search_time_s /= aConfig->noOfFullOrderQuery;

	printf("building time: %.10f\t%.10f\n", qc_tree_build_time_u,
			qc_tree_build_time_s);
	printf("querying time: %.10f\t%.10f\n", qc_tree_search_time_u,
			qc_tree_search_time_s);


	fprintf(timeFilePtr, "%.10f\t%.10f\n", qc_tree_build_time_u,
			qc_tree_build_time_s);
	fprintf(timeFilePtr, "%.10f\t%.10f\n", qc_tree_search_time_u,
			qc_tree_search_time_s);
#endif
}

/**
 * It generate a set of pointID of all points in MDC table
 * @param mdc The minimum disqualifying condition
 * @return A set of pointID of all points in MDC table
 * @author Terry & Sing
 * @since 27 Oct 2006
 */
mySet* getAllPointInMdc(myClassSetSetSet *mdc)
{
	int i=0;
	mySet *allMdcPoint;
	allMdcPoint = set_new();
	for (i=0; i< mdc->noOfElt; i++)
	{
		set_insert(allMdcPoint, mdc->pointID[i]);
	}
	return allMdcPoint;
}

/**
 * It generate a set of pointID of all data point
 * in raw data
 * @param noOfData Total number of data in raw data set
 * @return A set of pointID of all data point in raw data
 * @author Terry & Sing
 * @since 27 Oct 2006
 */
mySet* getSetOfAllDataPoint(int noOfData)
{
	int i=0;
	mySet *aSet;
	aSet = set_new();
	for (i=0; i<noOfData; i++)
	{
		set_insert(aSet, i);
	}
	return aSet;
}

/**
 * It find the set of pointID of skyline points
 * after a specified refinement.
 * @param aConfig Contain all raw data, including refinement
 * @param root Root node of the ready made MDC compressed tree
 * @param R
 * @return The set of skyline point after refinement
 * @author Terry & Sing
 * @since 27 Oct 2006
 */
mySet* mdcTree_findSkyline(config *aConfig, mdcTreeNode *root, 
mainOrdering* R)
{
	//int i;
	mySet* result = set_new();
	//mySet* disqPt = set_new();

//	anOrderSet = config_readOneQuery(filePtr, noOfClass, aConfig->noOfCat,
//			aConfig->noOfDim);
//	R = mainOrdering_transformOrderSet2MainOrdering(anOrderSet);
	int noOfNumAtrr;
	noOfNumAtrr = aConfig->noOfDim - aConfig->noOfCat;

	//printf("# disq pt at root: %d\n", root->disqualifiedPoint->noOfElt);
	//find all the disqualified point from the mdc tree
	//mdcTree_findDescendant(R, root, disqPt, noOfNumAtrr);
	//int *isSkyline;
	//isSkyline = (int *)malloc(sizeof(int)*aConfig->noOfData);
	//memset(isSkyline, 0, sizeof(int)*aConfig->noOfData);
	mdcTree_findDescendant(R, root, result, noOfNumAtrr);
	/*
	for (i=0; i<aConfig->noOfData; i++){
		if(isSkyline[i]==1)
		{
			set_insert(result, i);
		}
	}
	free(isSkyline);
	*/
//	printTotalOrder(aConfig, anOrderSet);
	return result;

	//return scanOut_skyline(disqPt, aConfig->noOfData);
}

/**
 * Algorithm to find set of disqualified points according
 * to a MDC compressed tree and refinement conditions at a tree
 * node.
 * But no use of set functions
 * @param R Refinement conditions(in query)
 * @param node Current tree node of MDC compressed tree
 * @param disqPt Accumulative set of disqualified points
 * @param noOfNumAtrr Number of numerical attribute
 * @author Terry & Sing
 * @since 27 Oct 2006
 */
void mdcTree_findDescendant(mainOrdering *R, mdcTreeNode *node,
		mySet* result, int noOfNumAtrr)
{
	mdcTreeNode *child;
	myClassSet *tempClass;

	//printf("b");
	//fflush(stdout);
	int i, j;

	if(node->parent==NULL)
	{
	//root store the candidate skyline pts
	/*
		for (i=0; i<node->disqualifiedPoint->noOfElt; i++)
		{
			isSkyline[node->disqualifiedPoint->elt[i]] = 1;
		}
		*/
		set_copy(result, node->disqualifiedPoint);
	}
	else
	{
	//update the disqualified point set
	/*
		for (i=0; i<node->disqualifiedPoint->noOfElt; i++)
		{
			isSkyline[node->disqualifiedPoint->elt[i]] = 0;
		}
		*/
		for (i=0; i<node->disqualifiedPoint->noOfElt; i++)
		{
			for (j=0; j<result->noOfElt; j++)
			{
				if (node->disqualifiedPoint->elt[i]==result->elt[j])
				{
					set_remove(result, j);
					break;
				}
			}
		}
	}
	
	//search for anymore possible path
	for (i=0; i<node->noOfChild; i++)
	{
	
		noOfVisitedNode++;
		child = node->child[i];

		tempClass = classSet_new();
		classSet_insert(tempClass, child->dimNo, child->classBefore);
		classSet_insert(tempClass, child->dimNo, child->classAfter);

		if(mainOrdering_isContain_disqualify(noOfNumAtrr, R, tempClass)==1)
		{
			mdcTree_findDescendant(R, child, result, noOfNumAtrr);
		}
		classSet_free(tempClass);
		free(tempClass);
	}
}


/**
 * Check if an ordering in Y is consist with those in Prefix
 * if (a, b) is inside Prefix, (b, a) would be removed from Y
 * @param y all the ordering has not explored yet
 * @param index the index of ordering in Y that going to check
 * @param prefix the ordering has be chosen
 * @return 1 if element in Y is inconsistent with Prefix
 * @author Terry
 * @since 15 Nov 2006
 */
int isInconsist(myClassSet* y, int index, myClassSet* prefix)
{
	int i;
	for(i=0; i<prefix->noOfElt; i++)
	{
		if(y->dimNo[index]==prefix->dimNo[i])
		{
			if (y->list[index]->elt[0] == prefix->list[i]->elt[1]
				&& y->list[index]->elt[1] == prefix->list[i]->elt[0])
			{
				return 1;
			}
		}
	}
	return 0;
}


/**
 * Update the prefix when a new ordering is chosen
 * @param prefix Prefix
 * @param y The ordering chosen
 * @param index
 * @return Updated prefix
 * @author Terry
 * @since 16 Nov 2006
 */
myClassSet* mdcTree_updatePrefix(myClassSet* prefix, myClassSet* y, int index)
{
	int i;
	int dimNo;
	int a;
	int b;


	myClassSet* newPrefix = classSet_new();
	classSet_copy(newPrefix, prefix);

	
	dimNo = y->dimNo[index];
	a = y->list[index]->elt[0];
	b = y->list[index]->elt[1];
	for(i=0; i<prefix->noOfElt; i++)
	{
		if(prefix->dimNo[i]==dimNo)
		{
			mySet* oldOrder = prefix->list[i];
			if(oldOrder->elt[1]==a)
			{
				mySet* newOrder = set_new();
				set_insert(newOrder, 0);
				set_insert(newOrder, 0);
				newOrder->elt[0]=oldOrder->elt[0];
				newOrder->elt[1]=b;
				classSet_insertSet(newPrefix, dimNo, newOrder);
			}
		}
	}
	classSet_insertSet(newPrefix, dimNo, y->list[index]);
	return newPrefix;
}


/**
 * Find out the from all points by removing those disquaified
 * @param disqPt set of disqualified points
 * @param noOfData number of the points
 * @author Terry
 * @since 18 Nov 2006
 */
mySet* scanOut_skyline(mySet* disqPt, int noOfData)
{
	mySet* result = set_new();

	int i;
	int *isMark;
	isMark = (int *)malloc(sizeof(int)*noOfData);
	memset(isMark, 0, sizeof(int)*noOfData);
	for(i=0;i<disqPt->noOfElt;i++)
	{
		isMark[disqPt->elt[i]]=1;	
	}
	for(i=0;i<noOfData;i++){
		if(isMark[i]!=1)
		{
			set_insert(result, i);
		}
	}

	return result;
}

/**
 * It read multiple query
 * @param filePtr File handle to read query from
 * @return number of query read
 * @author Sing
 */
int longQueryInit(FILE ** filePtr)
{
	int noOfQuery = 0;
	int noOfCat = 0;
	char *longQueryFile = "./experiment/query.long.500k.2d.txt";
	*filePtr = (FILE *) fopen(longQueryFile, "r");
	if (filePtr == NULL)
	{
		printf("File %s does not exist!\n", longQueryFile);
		exit(0);
	}
	fscanf(*filePtr, "%d\n", &noOfQuery);
	fscanf(*filePtr, "%d\n", &noOfCat);
	printf("No of query: %d\n", noOfQuery);
	return noOfQuery;
}

/**
 * Print out the total ordering in anorderSet
 * @param aConfig Config
 * @param anOrderSet
 * @author Terry & Sing
 * @since 21 Nov 2006
 */
void printTotalOrder(config *aConfig, myOrderSet *anOrderSet)
{
	int i, j;
	puts("");
	for (j = 0; j < aConfig->noOfCat; j++)
	{
		printf(" ordering (dim %d): ", anOrderSet->dimNo[j]);
		for (i = 0; i < anOrderSet->orderList[j]->noOfElt; i++)
		{
			printf("%d ", anOrderSet->orderList[j]->elt[i]);
		}
		printf("\n");
	}
}

/**
 * To calcualate the storeage size of the MDC tree with root
 * node. Please  note that it calcualte conceptual size 
 * rather than real size. e.g. pointer variable is not included
 * @param node Pointer to root node
 * @return size of MDC tree in byte
 * @author Terry & Sing
 * @since 21 Nov 2006
 */
int calculateSizeOfMdcCompressTree(mdcTreeNode *node)
{
	int size = 0;
	int i;
	//size of refinement condition
	size += 2 * sizeof(int);
	//size of set of disqualified points
	size += node->disqualifiedPoint->noOfElt * sizeof(int); 
	if (node->noOfChild > 0) {
		for (i = 0; i< node->noOfChild; i++) 
		{
			size += calculateSizeOfMdcCompressTree(node->child[i]);
		}
	}
	return size;
}

/**
 * Used in sorting the diqualifiying conditions using qsort.
 * @param p1 MDC compress tree node
 * @param p2 MDC compress tree node
 * @return 1 if p1 > p2, 0 if p1 = p2, -1 if p2 > p1
 * @author Terry
 * @since 29/11/2006
 */
int dimordercompare(const void *p1, const void *p2)
{
	int a1 = (*(mdcNode**)p1)->dimNo;
	int a2 = (*(mdcNode**)p1)->classAfter;
	int a3 = (*(mdcNode**)p1)->classBefore;
	int b1 = (*(mdcNode**)p2)->dimNo;
	int b2 = (*(mdcNode**)p2)->classAfter;
	int b3 = (*(mdcNode**)p2)->classBefore;

	if(a1>b1) {
		return 1;
	} else if(a1==b1) {
		if(a2>b2) {
			return 1;
		} else if(a2==b2) {
			if(a3>b3) {
				return 1;
			}else if(a3==b3){
					return 0;
			}else{
				return -1;
			}
		} else {
			return -1;
		}
	} else {
		return -1;
	}
}


/**
 * Built the mdc tree
 * @param root Root node of MDC compress tree to be build
 * @param mdc MDC table
 * @author Terry
 * @since 29/11/2006
 */
void buildMdcTree(mdcTreeNode* root, myClassSetSetSet* mdc)
{
	int i, j, k, m;
	int found;
	mdcTreeNode* currNode;

	int noOfOrder;
	myClassSet* aDisqCon;
	mdcNode** mdcN;

	//for each point
	for(i=0;i<mdc->noOfElt;i++)
	{
		//insert into root: the point could be skyline
		set_insert(root->disqualifiedPoint, mdc->pointID[i]);

		//for each disqualify condition
		for (j=0;j<mdc->elt[i]->noOfElt;j++)
		{
			
			aDisqCon = mdc->elt[i]->elt[j];
			noOfOrder = aDisqCon->noOfElt;

			//conver the class set into a mdcNode array
			
			mdcN = (mdcNode **)malloc(sizeof(mdcNode*)*noOfOrder);
			for (k=0;k<aDisqCon->noOfElt;k++)
			{
				mdcN[k] = (mdcNode *)malloc(sizeof(mdcNode));	
				mdcN[k]->dimNo = aDisqCon->dimNo[k];
				mdcN[k]->classBefore = aDisqCon->list[k]->elt[0];
				mdcN[k]->classAfter = aDisqCon->list[k]->elt[1];
			}

			//sort the ordering
			qsort((void *)mdcN, noOfOrder,
				sizeof(mdcNode*), dimordercompare);
			//debug
			//printf("order: ");
			//for (k=0;k<aDisqCon->noOfElt;k++)
			//{
			//	printf("[%d] %d<%d, ", mdcN[k]->dimNo,
			//		mdcN[k]->classBefore, mdcN[k]->classAfter);
			//}
			//printf("\n");


			//find the go along the tree
			currNode = root;
			found = 1;
			k = 0;
			while(k<noOfOrder)
			{	
				int x1, x2, x3;
				x1 = mdcN[k]->dimNo;
				x2 = mdcN[k]->classBefore;
				x3 = mdcN[k]->classAfter;
				found = 0;
				for(m=0;m<currNode->noOfChild;m++)
				{
					if(x1==currNode->child[m]->dimNo &&
						x2==currNode->child[m]->classBefore&&
						x3==currNode->child[m]->classAfter)
					{
						currNode = currNode->child[m];
						found = 1;
						break;
					}
				}
				if(found==1)
				{
					k++;
				}
				else
				{
					break;
				}
			}

			//add node when necessary
			while(k<noOfOrder)
			{
				//Create new node
				mdcTreeNode* newNode = mdcTreeNode_new(currNode, mdcN[k]->dimNo,
						mdcN[k]->classBefore, mdcN[k]->classAfter);
				mdcTreeNode_appendChild(currNode, newNode);
				currNode = newNode;
				noOfNode++;
				k++;
			}
			//insert pt
			set_insert(currNode->disqualifiedPoint, mdc->pointID[i]);
			
			//clear up
			for (k=0;k<aDisqCon->noOfElt;k++)
			{
				free(mdcN[k]);
			}
			free(mdcN);

		}
	}
}
