#include "set.h"
#include "ordering.h"
#include "config.h"
#ifndef WIN32
#include <sys/resource.h>
#include <sys/times.h>
#include <unistd.h>
#endif

/**
 * A node of minimun disqualifying condition
 * compression tree.
 */
typedef struct mdcTreeNode_s
{
	/** Set of pointID of disqualified point due to condition
	 * in label */
	mySet *disqualifiedPoint;
	/** Number of child node */
	int noOfChild;
	/** Parent node. NULL if it is a root node */
	struct mdcTreeNode_s *parent;
	/** Array of pointer to child node */
	struct mdcTreeNode_s **child;
	/**
	 * The dimension of refinement 
	 * -1 indicate no refinement
	 */
	int dimNo;
	/**
	 * The integer representation of A1 in refinement: (A1, A2)
	 * -1 indicate no refinement
	 */
	int classBefore;
	/** 
	 * The integer representation of A2 in refinement: (A1, A2)
	 * -1 indicate no refinement
	 */
	int classAfter;
} mdcTreeNode;

typedef struct mdcNode_s
{
	int dimNo;
	int classBefore;
	int classAfter;
} mdcNode;

myClassSet* getY(myClassSetSetSet* disqualifySet);
void mdcTreeNode_appendChild(mdcTreeNode *parent, mdcTreeNode *child);
mdcTreeNode* mdcTreeNode_new(mdcTreeNode *parent, int dimNo,
		int classBefore, int classAfter);
void expandTree(mdcTreeNode *node, myClassSetSetSet *mdc,
		myClassSet *y, myClassSet *prefix);
//void expandTreeInit(myClassSetSetSet *mdc, config *aConfig);
void expandTreeInit(myClassSetSetSet *mdc, config *aConfig, FILE *debugFilePtr, FILE *timeFilePtr);
void printNodeInfomation(FILE *filePtr, mdcTreeNode *node);
void printMdcCompressTree(FILE *filePtr, mdcTreeNode *node, int *id);
mySet* getAllPointInMdc(myClassSetSetSet *mdc);
mySet* getSetOfAllDataPoint(int noOfData);
mySet* mdcTree_findSkyline(config *aConfig, mdcTreeNode *root,
		mainOrdering* anOrderSet);
void mdcTree_findDescendant(mainOrdering *R, mdcTreeNode *node,
		mySet* skyline, int noOfNumAtrr);
int isInconsist(myClassSet* y, int index, myClassSet* prefix);
myClassSet* mdcTree_updatePrefix(myClassSet* prefix, myClassSet* y, int index);
mySet* scanOut_skyline(mySet* disqPt, int noOfData);
void calculateExecutionTime(struct rusage *myTime1, struct rusage *myTime2, float *userTime, float *sysTime);
int longQueryInit(FILE ** filePtr);
void printTotalOrder(config *aConfig, myOrderSet *anOrderSet);
int calculateSizeOfMdcCompressTree(mdcTreeNode *node);
int dimordercompare(const void *p1, const void *p2);
void buildMdcTree(mdcTreeNode* root, myClassSetSetSet* mdc);
