#ifndef RTREE_H
#define RTREE_H

#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <math.h>

#include <string.h>
#include "set.h"

#define CONFIG_FILE	"rtree.config"
//#define SAVE_RTREE_FILE "save_rtree_file"

#define FALSE    	0
#define TRUE     	1
#define ASC_NUM  	48
#define NO_ID	 	-1
#define FOUND		1
#define NOT_FOUND 	0

#define ROOT  0   
#define LEAF  1
#define NODE  2

#define INFINITY 2500 // for d_max in NN, equals ((highest dim)*(max dist))^2
#define UNDEFINED -3  // for id of entries in PR


/* Global variable ******************
m: min. number entries of each node;
M: max. number entries of each node;
dim: dimension of the rtree data.
*************************************/

int m, M, dim, reinsert_p, no_histogram;

#ifndef FILENAME_MAX
#define FILENAME_MAX 1024
#endif
char SAVE_RTREE_FILE[FILENAME_MAX];

int extra_level;

typedef struct node { float *a;
                      float *b;
		      int id;
                      int attribute;
                      int vacancy;
					  myClassSet *classGroup;
                      struct node *parent;
                      struct node **ptr; }   node_type;

typedef struct NN {	double dist;
			int oid;
			struct node *pointer; 
			struct NN *next; } NN_type;

typedef struct BranchArray {	double min;
				node_type *node;} ABL;


typedef struct configR_S { int dim;
			int m;
			int M;
			int reinsert_p;
			int no_histogram;
			char nodefile[FILENAME_MAX];
			char rootfile[FILENAME_MAX];
			char queryfile[FILENAME_MAX];
			char positionfile[FILENAME_MAX]; }   configR_type;

typedef struct rtreeGroup_s
{
	node_type **rtreeList;
	int noOfRtree;
} rtreeGroup;

void tree_node_allocate(node_type **node);

rtreeGroup *rtreeGroup_new();
//void insert_node(node_type *root, float *data, int seq_id, int classID);
void discard_rtreeGroup(rtreeGroup *anRtreeGroup);

//void insert_node(node_type *root, float *data, int seq_id, int dimNo, int classID);
void insert_node(node_type *root, float *data, int seq_id, myClassSet *projDimClass);
int count_rtreeSize(node_type **root);

#endif

